<?php
/**
 * NOTICE OF LICENSE
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 * @author	  Vladimir Turek
 * @copyright 2010-2020 PrestaAddons
 * @license   LICENSE_EN.txt
 */

if (!defined('_PS_VERSION_'))
	exit;

class M4Check extends Module
{
    const REMOVE = 'REMOVE';
    const XSAM = 'XsamXadoo Malware';
    const HAXOR = 'Marouane-Haxor';
    const DOORWAY = 'Spam Doorway Manager';

    private $checkDirs = array(
        'phpunit' => self::REMOVE,
    );
    private $checkAddedFiles = array(
        'XsamXadoo_Bot.php' => self::XSAM,
        'XsamXadoo_deface.php' => self::XSAM,
        '0x666.php' => self::XSAM,
        'f.php' => self::XSAM,
        'eval-stdin.php' => self::XSAM,
        'Xsam_Xadoo.html' => self::XSAM,
        'xsamxadoo.php' => self::XSAM,
        'xsamxadoo1.php' => self::XSAM,
        'haxor.php' => self::HAXOR,
        'h4x0r.html' => self::HAXOR,
        'haxor.html' => self::HAXOR,
        'phantom.html' => self::HAXOR,
        'new_readme.php' => self::DOORWAY,
        '404.php' => self::DOORWAY,
        'wp-log.php' => self::DOORWAY,
    );
    private $checkPrestaShopFiles = array(
        '/index.php' => array(
            'kind' => self::DOORWAY,
            'imprint' => 'smrequest_uri',
        ),
        '/.htaccess' => array(
            'kind' => self::DOORWAY,
            'imprint' => 'index.php?smsite',
        ),
    );

    function __construct()
    {
        $this->name = 'm4check';
        $this->tab = 'administration';
        $this->version = '1.1.0';
		$this->author = 'PrestaAddons';
		$this->need_instance = 0;
        $this->bootstrap = true;

        parent::__construct();

		$this->displayName = $this->l('M4 Check for vulnerabilities');
        $this->description = $this->l('Check for known vulnerabilities in your shop.');
        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
    }

    public function getContent()
    {
        return $this->renderForm();
    }

    public function renderForm()
    {
        $vulnerabilities = $this->postProcess();

        $this->context->smarty->assign(array(
            'vulnerabilities' => $vulnerabilities,
            'tested' => is_array($vulnerabilities),
        ));

        return $this->display(__FILE__, 'm4check.tpl');
    }

    public function postProcess()
    {
        if (Tools::isSubmit('submitCheck')) {
            return $this->scan(_PS_ROOT_DIR_, $this->checkDirs, $this->checkAddedFiles, $this->checkPrestaShopFiles);
        }
    }

    private function scan($root, $checkDirs, $checkAddedFiles, $checkPrestaShopFiles)
    {
        $dirs = array();
        $files = array();
        $prestashopFiles = array();

        $items = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($root), RecursiveIteratorIterator::CHILD_FIRST);

        foreach ($items as $splFileInfo) {
            $relativePath = str_replace($root, '', $splFileInfo->getPathname());

            if ($splFileInfo->isDir() && array_key_exists($splFileInfo->getFilename(), $checkDirs)) {
                $dirs[] = $relativePath.sprintf(' (%s)', $this->replaceMessage($checkDirs[$splFileInfo->getFilename()]));

            } elseif (array_key_exists($splFileInfo->getFilename(), $checkAddedFiles)) {
                $files[] = $relativePath.sprintf(' (%s)', $this->replaceMessage($checkAddedFiles[$splFileInfo->getFilename()]));

            } elseif (array_key_exists($relativePath, $checkPrestaShopFiles)) {
                $content = file_get_contents($splFileInfo->getPathname());
                if (strpos($content, $checkPrestaShopFiles[$relativePath]['imprint']) !== false) {
                    $prestashopFiles[] = str_replace($root, '', $splFileInfo->getPathname()) . sprintf(' (%s)', $this->replaceMessage($checkPrestaShopFiles[$relativePath]['kind']));
                }
            }
        }

        return array(
            'dirs' => $dirs,
            'files' => $files,
            'prestashopFiles' => $prestashopFiles,
        );
    }

    private function replaceMessage($message)
    {
        if ($message == self::REMOVE) {
            return $this->l('you should remove this directory in production web site');
        }

        return $message;
    }
}

